@@ -701,4 +701,9 @@ qn <- function(treat, subclass, discarded = NULL) {
 #Faster diff()
 diff1 <- function(x) {
   x[-1] - x[-length(x)]
-}
\ No newline at end of file
+}
+
+#Check if is a whole number
+is_whole_number <- function(x) {
+  abs(x - round(x)) < .Machine$double.eps
+}
