@@ -467,7 +467,6 @@ matchit2nearest <-  function(treat, data, distance, discarded,
   else {
     ratio <- as.integer(rep(ratio, n1))
   }
-  max_rat <- max(ratio)
 
   m.order <- {
     if (is.null(distance)) match_arg(m.order, c("data", "random"))
@@ -482,12 +481,15 @@ matchit2nearest <-  function(treat, data, distance, discarded,
                   "smallest" = order(distance[treat == 1], decreasing = FALSE),
                   "random" = sample.int(n1),
                   "data" = seq_len(n1))
-    mm <- nn_matchC(treat, ord, ratio, max_rat, discarded, reuse.max, distance, distance_mat, ex, caliper.dist,
-                    caliper.covs, caliper.covs.mat, mahcovs, antiexactcovs, unit.id, verbose)
+
+    mm <- nn_matchC_dispatch(treat, ord, ratio, discarded, reuse.max, distance, distance_mat,
+                             ex, caliper.dist, caliper.covs, caliper.covs.mat, mahcovs,
+                             antiexactcovs, unit.id, verbose)
+
   }
   else {
     distance_ <- caliper.covs.mat_ <- mahcovs_ <- antiexactcovs_ <- distance_mat_ <- NULL
-    mm_list <- lapply(levels(ex), function(e) {
+    mm_list <- lapply(levels(ex)[cc], function(e) {
       if (verbose) {
         cat(sprintf("Matching subgroup %s/%s: %s...\n",
                     match(e, levels(ex)), nlevels(ex), e))
@@ -514,15 +516,23 @@ matchit2nearest <-  function(treat, data, distance, discarded,
                      "random" = sample.int(n1_),
                      "data" = seq_len(n1_))
 
-      mm_ <- nn_matchC(treat_, ord_, ratio_, max_rat, discarded_, reuse.max, distance_, distance_mat_, NULL, caliper.dist,
-                       caliper.covs, caliper.covs.mat_, mahcovs_, antiexactcovs_, NULL, verbose)
+      mm_ <- nn_matchC_dispatch(treat_, ord_, ratio_, discarded_, reuse.max, distance_, distance_mat_,
+                               NULL, caliper.dist, caliper.covs, caliper.covs.mat_, mahcovs_,
+                               antiexactcovs_, NULL, verbose)
 
       #Ensure matched indices correspond to indices in full sample, not subgroup
       mm_[] <- seq_along(treat)[.e][mm_]
       mm_
     })
 
-    mm <- do.call("rbind", mm_list)[lab1,, drop = FALSE]
+    #Construct match.matrix
+    mm <- matrix(NA_integer_, nrow = length(lab1),
+                 ncol = max(vapply(mm_list, ncol, numeric(1L))),
+                 dimnames = list(lab1, NULL))
+
+    for (m in mm_list) {
+      mm[rownames(m), seq_len(ncol(m))] <- m
+    }
   }
 
   if (verbose) cat("Calculating matching weights... ")
@@ -544,3 +554,20 @@ matchit2nearest <-  function(treat, data, distance, discarded,
 
   return(res)
 }
+
+# Dispatches Rcpp function for NN matching
+# nn_matchC_vec() if distance_mat and mahcovs are NULL
+# nn_matchC() otherwise
+nn_matchC_dispatch <- function(treat, ord, ratio, discarded, reuse.max, distance, distance_mat, ex, caliper.dist,
+                               caliper.covs, caliper.covs.mat, mahcovs, antiexactcovs, unit.id, verbose) {
+  if (is.null(distance_mat) && is.null(mahcovs)) {
+    nn_matchC_vec(treat, ord, ratio, discarded, reuse.max, distance,
+                  ex, caliper.dist, caliper.covs, caliper.covs.mat,
+                  antiexactcovs, unit.id, verbose)
+  }
+  else {
+    nn_matchC(treat, ord, ratio, discarded, reuse.max, distance, distance_mat,
+              ex, caliper.dist, caliper.covs, caliper.covs.mat, mahcovs,
+              antiexactcovs, unit.id, verbose)
+  }
+}
\ No newline at end of file
